﻿using System;
using System.Windows.Input;
using System.Diagnostics.Contracts;

namespace Framework.WPF
{
    /// <summary>
    /// A command whose sole purpose is to relay its functionality to other objects by invoking delegates.
    /// The default return value for the CanExecute method is 'true'.
    /// </summary>
    /// <remarks>
    /// Extrait de MSDN Mag Fév 2009. 
    /// http://msdn.microsoft.com/en-us/magazine/dd419663.aspx#id0090030
    /// </remarks>
    public class RelayCommand : ICommand
    {
        #region Fields
        private readonly Action<object> _execute;
        private readonly Predicate<object> _canExecute;
        #endregion

        #region Constructors
        /// <summary>
        /// Creates a new command that can always execute.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<object> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Creates a new command.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<object> execute, Predicate<object> canExecute)
        {
            // Contract nécessite le Framework .NET 4 et on n'est qu'en version 3.5 
            //Contract.Requires(execute != null);
            if (execute == null)
                throw new ArgumentNullException("execute");

            _execute = execute;
            _canExecute = canExecute;
        }
        #endregion

        #region ICommand Members
        /// <summary>
        /// Définit la méthode qui détermine si la commande peut s'exécuter dans son état actuel.
        /// </summary>
        /// <param name="parameter">Données utilisées par la commande.Si la commande ne requiert pas que les données soient passées, cet objet peut avoir la valeur null.</param>
        /// <returns>
        /// true si cette commande peut être exécutée ; sinon false.
        /// </returns>
        public bool CanExecute(object parameter)
        {
            return _canExecute == null ? true : _canExecute(parameter);
        }

        /// <summary>
        /// Se produit lorsque des modifications influent sur l'exécution de la commande.
        /// </summary>
        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        /// <summary>
        /// Définit la méthode à appeler lorsque la commande est appelée.
        /// </summary>
        /// <param name="parameter">Données utilisées par la commande.Si la commande ne requiert pas que les données soient passées, cet objet peut avoir la valeur null.</param>
        public void Execute(object parameter)
        {
            _execute(parameter);
        }
        #endregion

        #region Static Method
        /// <summary>
        /// Build a RelayCommand that sets the Cursor to Wait, executes the command, and turn the Cursor back to normal.
        /// Use for long tasks.
        /// </summary>
        /// <param name="execute">The execute.</param>
        /// <param name="canExecute">The can execute.</param>
        /// <returns></returns>
        public static RelayCommand BuildHourglassCommand(Action<object> execute, Predicate<object> canExecute)
        {
            Contract.Requires(execute != null);
            Action<object> waitingExecute = new Action<object>(param =>
            {
                Mouse.OverrideCursor = Cursors.Wait;
                execute.Invoke(param);
                Mouse.OverrideCursor = null;
            });
            if (canExecute != null)
            {
                return new RelayCommand(waitingExecute, canExecute);
            }
            else
            {
                return new RelayCommand(waitingExecute);
            }
        }

        /// <summary>
        /// Build a RelayCommand that sets the Cursor to Wait, executes the command, and turn the Cursor back to normal.
        /// Use for long tasks.
        /// </summary>
        /// <param name="execute">The execute.</param>
        /// <returns></returns>
        public static RelayCommand BuildHourglassCommand(Action<object> execute)
        {
            return BuildHourglassCommand(execute, null);
        }
        #endregion
    }
}
